
/*
 *  Copyright (c) 1998 - 1999 Karel Zak "Zakkr" <zakkr@zf.jcu.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 */
	
/* READER - flex source code */


/* ----------- flex definitions ------------- */
%{
	#include <stdlib.h>
	#include <string.h>
	#include <unistd.h>
	#include <sys/stat.h>

	#include "../src/config.h"
	
	#define	SET_STREND(x)	x[strlen(x)-1] = '\0'	
	#define YY_NO_UNPUT
	#define RE_ERROR	0
	#define RE_OK		1	
	
	typedef struct {
		char	name[50];
		char	file[10];
	} ConfP;

	static int 	kp_count=-1, 
			set=0;
	static FILE	*f_new;
	static ConfP	*prof;	
	static int	num;
	static int	defaultP;
	static char	confdir[256];
	static char	newfile[256];
	static char	oldfile[256];
%}

STRING		[^\"^\n]
STRING_END	[\"]
BLANK		[[:blank:]|\n]
NUMBER		[[:digit:]]

/* start conditions */
%s		profile_name in_file profile_num

/* ----------- flex rules ------------------- */
%% 

"PROFILE_NAME"+([\"]|{BLANK}+[\"])	BEGIN(profile_name);
"IN_FILE"+([\"]|{BLANK}+[\"])		BEGIN(in_file);
"PROFILE_NUMBER"+([\"]|{BLANK}+[\"])	BEGIN(profile_num);
			
<profile_num>{NUMBER}+{STRING_END} { 
			SET_STREND(yytext);
			num = atoi(yytext);
			if (kp_count != -1) {
	 			fputs("conf2ini: Fatal error of configure data\n", stderr);
	 			exit(RE_ERROR);
      			}
			if ((prof=calloc(num+1, sizeof(ConfP))) == NULL) {
				fputs("conf2ini: Can't allocate memory\n", stderr);
	 			exit(RE_ERROR);		
			}
			BEGIN(INITIAL);			 	 
		}
<profile_name>{STRING}+{STRING_END} { 
			SET_STREND(yytext);
			if (kp_count < num) ++kp_count;
			else return RE_OK;
			strncpy(prof[kp_count].name, yytext, 50);
			BEGIN(INITIAL);			 	 
		}
<in_file>{NUMBER}+{STRING_END} { 
			SET_STREND(yytext);
			strncpy(prof[kp_count].file, yytext, 10);
			BEGIN(INITIAL);			 	 
		}
"default"	{
			if (set == 0 && kp_count > -1) {
				defaultP = kp_count;
				set = 1;		
			}
		}
.			/* ignore */
"\n"			/* ignore */
%%
/* ----------- the end of flex source code --- */

static int conf_loader(char *filename) 
{ 
	if ((yyin=fopen(filename,"r")) == NULL) {
		perror(filename);
		exit(RE_ERROR);
	}
	yylex();
	fclose(yyin);
	return RE_OK;	
}

void prof2new(char *filename, char *profname)
{
	char	p_oldfile[256];
	char	p_newfile[256];
	
	sprintf(p_oldfile, "%s/%s", confdir, filename);
	sprintf(p_newfile, "%s/%s", confdir, profname);	
	rename(p_oldfile, p_newfile);	
}

void print_ini()
{
	int	i;
	char	*loc;
	
     /* About */
     	fprintf(f_new, "\n[about]\n");
     	fprintf(f_new, "kim_version=%s\n", KIM_VERSION);
     /*	Layout */
	fprintf(f_new, "\n[layout]\n");
	fprintf(f_new, "color_zombie=1\ncolor_run=1\n");
	fprintf(f_new, "save=0\n");
	fprintf(f_new, "sort=0\n");
	fprintf(f_new, "sort_cols=0\n");
	fprintf(f_new, "view=0\n");
	fprintf(f_new, "inpage=0\n");
	fprintf(f_new, "infoline=0\n");
      /* Profiling */
	fprintf(f_new, "\n[profiling]\n");
	fprintf(f_new, "num=%d\n", num);
	fprintf(f_new, "default=%d\n", defaultP);
	
	for (i=0; i<=num; i++) {
		if ((loc = strchr(prof[i].name, ' ')) != NULL) {
			do {
				*loc = '_';
				loc = strchr(prof[i].name, ' ');
			} while (loc);
		}
		fprintf(f_new, "profile_name=%s\n", prof[i].name);
		prof2new(prof[i].file, prof[i].name);
	}
}

int main(int argc, char **argv)
{
	struct stat	st;

	fprintf(stderr, "\nKim conf2ini: ");
	
	if (argc >= 2) 
		strncpy(confdir, argv[1], 255);
	else {
		fprintf(stderr, "Bad arg. !\n\n");
		exit( RE_ERROR );	 
	}
	
	sprintf(newfile, "%s/%s", confdir, KIM_INIFILE);   
	sprintf(oldfile, "%s/%s", confdir, KIM_OLD_CONF);   
	if (stat(newfile, &st) == 0) {
		fprintf(stderr, "conf2ini: file %s exist !\n\n", newfile); 	
		exit(RE_ERROR);
	}
	if ((f_new=fopen(newfile, "w")) == NULL) {
		perror(newfile);
		exit (RE_ERROR);
	}
	conf_loader(oldfile);
	print_ini();
	
	fclose(f_new);
	unlink(oldfile);
	
	fprintf(stderr, "Translation from old KIM config version to new '.ini' format is finished.\n\n");  
		
	exit(RE_OK);	
}

